@@ -8,9 +8,26 @@ get.point3d <- function(atom) {
   }
   
   p3d <- .jcall(atom, "Ljavax/vecmath/Point3d;", "getPoint3d")
-  c(.jfield(p3d, name='x'),
-    .jfield(p3d, name='y'),
-    .jfield(p3d, name='z'))
+  if (is.jnull(p3d)) return( c(NA,NA,NA) )
+  else {
+    c(.jfield(p3d, name='x'),
+      .jfield(p3d, name='y'),
+      .jfield(p3d, name='z'))
+  }
+}
+
+get.point2d <- function(atom) {
+  if (is.null(attr(atom, 'jclass')) ||
+      attr(atom, "jclass") != "org/openscience/cdk/interfaces/IAtom") {
+    stop("Must supply an IAtom object")
+  }
+  
+  p3d <- .jcall(atom, "Ljavax/vecmath/Point2d;", "getPoint2d")
+  if (is.jnull(p3d)) return( c(NA,NA) )
+  else {
+    c(.jfield(p3d, name='x'),
+      .jfield(p3d, name='y'))
+  }
 }
 
 get.symbol <- function(atom) {
